<?php
/*
Plugin Name:Similar Posts
Plugin URI: http://rmarsh.com/plugins/similar-posts/
Description: Displays a list of posts similar to the current one by comparing the way they use words. <a href="http://rmarsh.com/plugins/post-options/">Instructions and help online.</a>
Version: 2.1.1 beta
Author: Rob Marsh, SJ
Author URI: http://rmarsh.com/
*/

/*
Copyright 2007  Rob Marsh, SJ  (http://rmarsh.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details: http://www.gnu.org/licenses/gpl.txt
*/


/*
	Template Tag: Displays the posts most similar to the current post.
		e.g.: <ul><?php similar_posts(); ?></ul>
	Full help and instructions at http://rmarsh.com/plugins/post-options/
*/
function similar_posts($args = '') {
	global $post;
	$result = false;
	// if the Plugin Output Cache plugin is installed we can cheat...
	if (defined('POC_CACHE')) {
		$key = 's_p'.$post->ID.$args;
		$cache = new POC_Cache();
		$cache->timer_start();
		$result = $cache->fetch($key);
		if ($result) $cache_time = sprintf('<!-- In fact it took %.3f seconds from the cache -->', $cache->timer_stop());
	}
	// ... otherwise we do it the hard way
	if (false === $result) {
		$rp = new SimilarPosts($args, '<li>{link}</li>');
		$rp->workhorse();
		$result = $rp->output();
		unset($rp);
		if (defined('POC_CACHE')) $cache->store($key, $result);
	} 
	echo $result;
	if (defined('POC_CACHE')) echo $cache_time;
}

/*
	Define the options menu
*/
function similar_posts_option_menu() {
	if (function_exists('current_user_can')) {
		if (!current_user_can('manage_options')) return;
	} else {
		global $user_level;
		get_currentuserinfo();
		if ($user_level < 8) return;
	}
	if (function_exists('add_options_page')) {
		add_options_page(__('Similar Posts Options'), __('Similar Posts'), 1, __FILE__, array('SimilarPosts', 'options_page'));
	}
}

function similar_posts_manage_menu($nothing) {
	if (function_exists('current_user_can')) {
		if (!current_user_can('manage_options')) return;
	} else {
		global $user_level;
		get_currentuserinfo();
		if ($user_level < 8) return;
	}
	if (function_exists('add_management_page')) {
		add_management_page(__('Similar Posts'), __('Similar Posts'), 1, __FILE__, 'similar_posts_manage_page');	
	}
}

// check to see if the fulltext index is in place
function check_full_text_index() {
	global $wpdb;
	//check to see if the index is already in place
	$spo_index_exists = FALSE;
	$index_info = $wpdb->get_results('SHOW INDEX FROM ' . $wpdb->posts, ARRAY_A);
	if (!$index_info) return FALSE;
	foreach ($index_info as $index) {
		if ($index['Key_name'] == 'post_similar') {
			$spo_index_exists = TRUE;
		}
	}
	return $spo_index_exists;
}

// this function gets called when the plugin is installed to set up the index and default options
function similar_posts_index_installation() {
   	global $wpdb;
	// Prepare the default set of options
	$default_options['limit'] = 5;
	$default_options['skip'] = 0;
	$default_options['trim_before'] = '';
	$default_options['show_private'] = 'false';
	$default_options['show_pages'] = 'false';
	$default_options['none_text'] = '<li>'.__('None Found').'</li>';
	$default_options['excluded_cats'] = '9999'; // '9999' coressponds to no categories excluded
	$default_options['excluded_authors'] = '9999'; // '9999' coressponds to no categories excluded
	$default_options['excluded_posts'] = ''; 
	$default_options['stripcodes'] = array(array());
	$default_options['prefix'] = '<ul>';
	$default_options['suffix'] = '</ul>';
	$default_options['output_template'] = '<li>{link}</li>';
	$default_options['match_cat'] = 'false';
	$default_options['bias_title'] = 'equal';
	$default_options['utf8'] = 'false';
	$default_options['num_terms'] = 20;
	// the plugin options are stored in the options table under the name of the plugin file sans extension
	add_option(basename(__FILE__, ".php"), $default_options, 'options for the Similar Posts plugin');
	
	if (!check_full_text_index()) {
		//try and set up the new index
		$wpdb->query('ALTER TABLE `'.$wpdb->posts.'` ADD FULLTEXT `post_similar` ( `post_title` ,' . ' `post_content` )');
	}
	
}

add_action('activate_Similar_Posts/similar-posts.php','similar_posts_index_installation');


// we can't run the plugin if the fulltext index is missing...
if (check_full_text_index()) {
	//the next lines find the language WordPress is using
	$language = WPLANG;
	//if no language is specified make it the default which is 'en'
	if ($language == '') {
		$language = 'en';
	}
	$overusedwordsfile = $language.'.words.php';
	//see if there is a file of overused words in that language in the right directory
	if (!file_exists($overusedwordsfile)) {
		//if not revert to the default file
		$overusedwordsfile = 'en.words.php';
	}
	//the file contains a single php command to create the $overusedwords array -- see 'en.words.php' for an example
	require_once($overusedwordsfile);
	
	//install the similarterm actions into WordPress
	add_action('save_post', 'save_similar_terms', 1);
	add_action('delete_post', 'delete_similar_terms', 1);
	if ($wp_db_version < 3308 ) { 
		add_action('edit_post', 'save_similar_terms', 1);
		add_action('publish_post', 'save_similar_terms', 1);
	} else {
		add_action('save_post', 'save_similar_terms', 1);
	}
	
	//install the admin pages
	add_action('admin_menu', 'similar_posts_option_menu');
	add_action('admin_menu', 'similar_posts_manage_menu');

} else {
	_e('Problem creating the full text index for Similar Posts. Please check the <a href="http://rmarsh.com/plugins/similar-posts/">instructions</a> on how to create the index manually.<br />'); 
}

/*
	The plugin class does all the work.
*/
class SimilarPosts {
	var $p; // the various parameters
	var $results; // the query results
	var $running_time;
	var $count_terms;
	
	// The constructor processes the various parameters
	function SimilarPosts($args, $default_output){
		// First we process any arguments to see if any defaults have been overridden
		// 	$args is of the form 'key1=val1&key2=val2'
		//	The code should cope with null values, e.g., 'key1=&key2=val2' 
		//	and arguments with embedded '=', e.g. 'output_template=<li class="stuff">{...}</li>'.
		$this->running_time = $this->getmicrotime();
		if($args){
			// separate the arguments into key=value pairs
			$arguments = explode("&", $args);
			foreach($arguments as $arg){
				if($arg){
					// find the position of the first '='
					$i = strpos($arg, '=');
					// if not a valid format ('key=value) we ignore it
					if ($i){
						$key = substr($arg, 0, $i);
						$val = substr($arg, $i+1); 
						// store the arguments in $this->p
						$this->p[$key]=$val;
					}
				}
			}
		}
		
		// This block retrieves the options stored in the database and uses them unless a value has been overridden 
		// by specifying it's value in the arguments above
		
		$options = get_option(basename(__FILE__, ".php"));
		
	  	if (!isset($this->p['limit'])) $this->p['limit'] = stripslashes($options['limit']);
	  	if (!isset($this->p['skip'])) $this->p['skip'] = stripslashes($options['skip']);
	  	if (!isset($this->p['trim_before'])) $this->p['trim_before'] = stripslashes($options['trim_before']);
	  	if (!isset($this->p['show_private'])) $this->p['show_private'] = $options['show_private'];
		if (!isset($this->p['show_pages'])) $this->p['show_pages'] = $options['show_pages'];
		if (!isset($this->p['none_text'])) $this->p['none_text'] = stripslashes($options['none_text']);
		if (!isset($this->p['excluded_cats'])) $this->p['excluded_cats'] = stripslashes($options['excluded_cats']);
		// a blank value has to be translated to the internal code '9999'
		if (trim($this->p['excluded_cats'])=='') $this->p['excluded_cats'] = '9999';
		if (!isset($this->p['excluded_authors'])) $this->p['excluded_authors'] = stripslashes($options['excluded_authors']);
		if (trim($this->p['excluded_authors'])=='') $this->p['excluded_authors'] = '9999';
		if (!isset($this->p['excluded_posts'])) $this->p['excluded_posts'] = stripslashes($options['excluded_posts']);
		if (!isset($this->p['stripcodes'])) $this->p['stripcodes'] = $options['stripcodes'];
	  	if (!isset($this->p['prefix'])) $this->p['prefix'] = stripslashes($options['prefix']);
	  	if (!isset($this->p['suffix'])) $this->p['suffix'] = stripslashes($options['suffix']);
	  	if (!isset($this->p['output_template'])) $this->p['output_template'] = stripslashes($options['output_template']);
		// an empty output_template makes no sense so we fall back to the default
		if ($this->p['output_template'] == '') $this->p['output_template'] = $default_output;
		
		// just for Similar Posts
		if (!isset($this->p['match_cat'])) $this->p['match_cat'] = $options['match_cat'];
		if (!isset($this->p['bias_title'])) $this->p['bias_title'] = $options['bias_title'];
		if (!isset($this->p['utf8'])) $this->p['utf8'] = $options['utf8'];
	  	if (!isset($this->p['num_terms'])) $this->p['num_terms'] = stripslashes($options['num_terms']);

		// Now we process the output_template to find the embedded tags which are to be replaced
		// with values taken from the database.
		
		// These are the tags which are recognised by the output template processor and 
		// the names of the database fields needed to evaluate each one.
		$valid_tags = array(
			'score' => array('ID'),
			'title' => array('post_title'),
			'url' => array('ID'),
			'author' => array(post_author),
			'date' => array('post_date'),
			'dateedited' => array('post_modified'),
			'excerpt' => array('post_excerpt', 'post_content'),
			'snippet' => array('post_content'),
			'snippetword' => array('post_content'),
			'fullpost' => array('post_content'),
			'categorylinks' => array('ID'),
			'catnames' => array('ID'),
			'commentcount' => array('comment_count'),
			'link' => array('post_title', 'ID', 'post_date'));
			
		// this will contain the needed fields
		$fields = array();
		$fields[] = 'ID';
		// Find all the tags in the template and store them in the $matches array
		preg_match_all('/{(.+?)}/', $this->p['output_template'], $matches);
		
		// A tag is of the form , {tag:ext}, where the tag part will be evaluated and replaced 
		// and the optional ext part provides extra data 
		
		// this will be the array of tags to replace
		$fulltags = array();
		// this will be the array of validated tags
		$tags = array();
		// this will be the array of extra data
		$exts = array();
		// get each tag
		foreach ($matches[0] as $match) {
			// get the extra data  (if any)
			$pos = strpos($match, ':');
			if ($pos) {
				$ext = rtrim(substr($match, $pos+1), '}');
				$tag = ltrim(substr($match, 0, $pos), '{');
			} else {
				$ext = '';
				$tag = trim($match, '{}');
			}
			// check if it is a valid tag
			if (isset($valid_tags[$tag])){
				// add the tag in a form ready to use in preg_replace later
				// the Q and E make sure that nothing in the tag is taken as a metacharacter
				$fulltags[] = '/\Q'.$match.'\E/';
				// the bare tag
				$tags[] = $tag;
				// the extra data if any
				$exts[] = $ext;
				// store the needed field names
				foreach ($valid_tags[$tag] as $fieldname) {
					$fields[] = $fieldname;
				}
			} 
		}
		
		// Store the tag information in the parameter array
		$this->p['fulltags'] = $fulltags;
		$this->p['tags'] = $tags;
		$this->p['exts'] = $exts;
		
		// remove any duplicate fields
		$this->p['fields'] = array_unique($fields);
	}

	// This method sets up and executes the database query that does all the work
	function workhorse() {
		global $wpdb, $wp_version, $post;
		if ($wp_version < 2.1) {
			$time_difference = get_settings('gmt_offset');
			$now = gmdate("Y-m-d H:i:s",(time()+($time_difference*3600)));
		}
		//set up bits of the query string
		if ($wp_version < 2.1) {
			$showpage = ($this->p['show_pages']=='true') ? "AND post_status IN ('publish', 'static') " : "AND post_status IN ('publish') ";
		} else {
			$showpage = ($this->p['show_pages']=='true') ? "AND post_status IN ('publish') AND post_type IN ('page', 'post') " : "AND post_status IN ('publish') AND post_type IN ('post') ";
		}	
		$showpass = ($this->p['show_private']=='false') ? "AND post_password ='' " : "";
		$excluded_cats = $this->p['excluded_cats'];
		$excluded_authors = $this->p['excluded_authors'];
		$excluded_posts = trim($this->p['excluded_posts']);
		$match_category = ($this->p['match_cat'] === 'true');
		$utf8 = ($this->p['utf8'] === 'true');
		if ($match_category) {
			$cat_ids = '';
			foreach(get_the_category() as $cat) {
				$cat_ids .= $cat->cat_ID . ',';
			}
			$cat_ids = rtrim($cat_ids, ',');
		}
		$limit = $this->p['skip'].', '.$this->p['limit'];
		$needed_fields = implode(', ', $this->p['fields']);
		
 		//get the terms to do the matching
		//see if there's a custom field stored for this post
		$postcustom = get_post_custom_values('similarterms');
		if (empty($postcustom)) {
			//if not generate the terms on the fly
			$terms = get_terms($post->post_content, $post->post_title, $utf8, $this->p['bias_title'], $this->p['num_terms']);
		} else {
			//use the stored terms
			$terms = addslashes($postcustom[0]);
		}
		$this->count_terms = substr_count($terms, ' ') + 1;

		// the workhorse...
		$sql = "SELECT DISTINCT " . $needed_fields . ", MATCH (post_title, post_content) AGAINST ( '$terms' ) as score "; 
		if (($excluded_cats != '9999') || ($match_category)){
			$sql .= " FROM $wpdb->posts LEFT JOIN $wpdb->post2cat ON ID = post_id ";
		} else {
			$sql .= " FROM $wpdb->posts ";
		}
		$sql .= "WHERE MATCH (post_title, post_content) AGAINST ( '$terms' ) ";
		if ($wp_version < 2.1) {
			$sql .= "AND post_date <= '$now' ";
		} else {
			$sql .= "AND post_status != 'future' ";
		}
		if ($match_category) {
			$sql .= "AND category_id IN ( $cat_ids ) ";
		}
		$sql .= $showpage;			
		if ($excluded_cats != '9999') {
			$sql .=	"AND (category_id NOT IN ( $excluded_cats ) && post_author NOT IN ( $excluded_authors ) ) ";
		} else {
			$sql .=	"AND post_author NOT IN ( $excluded_authors ) ";
		}	
		if ($excluded_posts != '') {
			$sql .= "AND ID NOT IN ( $excluded_posts ) ";
		}
		// don't match self
		$sql .= "AND ID != '$post->ID' ";
		$sql .= $showpass . "LIMIT $limit";
	    $this->results = $wpdb->get_results($sql);
	}
	
	function format_date($date, $fmt) {
			if ($fmt === '') $fmt = get_settings('date_format');
			return mysql2date($fmt, $date);
	}

	function regescape($s) {
			$s = str_replace('\\', '\\\\', $s);
			$s = str_replace('/', '\\/', $s);
			$s = str_replace('[', '\\[', $s);
			$s = str_replace(']', '\\]', $s);
			return $s;
	}

	function strip_special_tags($text, $stripcodes) {
			$numtags = count($stripcodes);
			for ($i = 0; $i < $numtags; $i++) {
				if (!$stripcodes[$i]['start'] || !$stripcodes[$i]['end']) return $text;
				$pattern = '/('. $this->regescape($stripcodes[$i]['start']) . '(.*?)' . $this->regescape($stripcodes[$i]['end']) . ')/i';
				$text = preg_replace($pattern, '', $text);
			}
			return $text;
	}

	function trim_excerpt($content, $len) {
		// taken from the wp_trim_excerpt filter
		$text = $content;
		$text = apply_filters('the_content', $text);
		$text = str_replace(']]>', ']]&gt;', $text);
		$text = strip_tags($text);
		if (!$len) $len = 55; 
		$excerpt_length = $len;
		$words = explode(' ', $text, $excerpt_length + 1);
		if (count($words) > $excerpt_length) {
			array_pop($words);
			$text = implode(' ', $words);
		}
		$text = convert_smilies($text);
		return $text;
	}
	
	function format_snippet($content, $p, $trim, $len) {
		$content = strip_tags($content);
		$content = $this->strip_special_tags($content, $this->p['stripcodes']);
		$content = stripslashes($content);
		// grab a maximum number of characters
		$snippet = substr($content, 0, $len);
		if ($trim == 'word' && strlen($snippet) == $len) {
			// trim back to the last full word--NB if our snippet ends on a word
			// boundary we still have to trim back to the non-word character
			// (the final 's' in the pattern makes sure we match newlines)
			preg_match('/^(.*)\W/s', $snippet, $matches);
			//if we can't get a single full word we use the full snippet
			// (we use $matches[1] because we don't want the white-space)
			if ($matches[1]) $snippet = $matches[1];
		}
		return $snippet;
	}

	function output() {
		global $wpdb;
		$output = ''; 
	    if ($this->results) {
			foreach ($this->results as $result) {
				$values = array();
				$numtags = count($this->p['tags']);
				for ($i = 0; $i < $numtags; $i++) {
					$tag = $this->p['tags'][$i]; 
					$ext = $this->p['exts'][$i];
					switch ($tag) {
					case 'score':
						$value = sprintf("%.0f", 37.0 * $result->score / $this->count_terms);	
						break;
					case 'title':
						$value = apply_filters('the_title', $result->post_title);	
						break;
					case 'url':
						$value = get_permalink($result->ID);
						break;
					case 'author':
						if (!$authordata) { $authordata = get_userdata($result->post_author); }
						$value = get_the_author();
						break;
					case 'date':
						$value = $this->format_date($result->post_date, $ext);
						break;
					case 'dateedited':
						$value = $this->format_date($result->post_modified, $ext);
						break;
					case 'excerpt':
						$value = trim($result->post_excerpt) ;
						if ($value == '') $value = $result->post_content;
						$value = $this->trim_excerpt($value, $ext);
						break;
					case 'snippet':
						if (!$ext) $ext = 100;
						$value = $this->format_snippet($result->post_content, $p, 'char', $ext);
						break;
					case 'snippetword':
						if (!$ext) $ext = 100;
						$value = $this->format_snippet($result->post_content, $p, 'word', $ext);
						break;
					case 'fullpost':
						$value = apply_filters('the_content', $result->post_content);
						break;
					case 'commentcount':
						$value = $result->comment_count;
						if ($ext) {
							$s = explode(':', $ext);
							if (count($s) == 3) {
								if ($value == 0) $value = $s[0];
								elseif ($value == 1) $value .= ' ' . $s[1];
								else $value .= ' ' . $s[2];
							}
						}
						break;
					case 'categorylinks':
						$cats = get_the_category($result->ID);
						$value = ''; $n = 0;
						foreach ($cats as $cat) {
							if ($n > 0) $value .= $ext;
							$value .= '<a href="' . get_category_link($cat->cat_ID) . '" title="' . sprintf(__("View all posts in %s"), $cat->cat_name) . '" rel="category tag">'.$cat->cat_name.'</a>';
							++$n;
						}
						break;
					case 'catnames':
						$cats = get_the_category($result->ID);
						$value = ''; $n = 0;
						foreach ($cats as $cat) {
							if ($n > 0) $value .= $ext;
							$value .= $cat->cat_name;
							++$n;
						}
						break;
					case 'link':
						$ttl = stripslashes(apply_filters('the_title', $result->post_title));
						$pml = get_permalink($result->ID);
						$pdt = $this->format_date($result->post_date, $ext);
						$value = "<a href=\"$pml\" rel=\"bookmark\" title=\"$pdt\">$ttl</a>";
						break;
					default:
						// does not match our list of tags
						die('an unrecognised tag has been detected in the output template');
					}
					$values[] = $value;
				}
				// Replace every valid tag with its value
				$output .= preg_replace($this->p['fulltags'], $values, $this->p['output_template'], 1);
				$output .= "\n";
			} 
			// If stuff is to be trimmed off the front...
			if ($this->p['trim_before']!=='') {
				// check it is actually there...
				if (strpos($output, $this->p['trim_before']) === 0) {
					// then snip it
					$output = substr($output, strlen($this->p['trim_before']));
				}
			}
			// finally display it all
			return $this->p['prefix'] . $output . $this->p['suffix'] . sprintf("<!--Similar Posts took %.3f seconds-->", $this->getmicrotime() - $this->running_time);
		} else {
			// or display the blank message
	        return $this->p['prefix'] . $this->p['none_text'] . $this->p['suffix'] . sprintf("<!--Similar Posts took %.3f seconds-->", $this->getmicrotime() - $this->running_time);
		}
	}
	
	function getmicrotime() {
	    // split output from microtime() on a space
	    list($usec, $sec) = explode(" ", microtime());
	    // append in correct order
	    return ((float)$usec + (float)$sec); 
	} 

	// This method displays, stores and updates all the options
	function options_page(){
		global $wpdb;
		// This bit stores any updated values when the Update button has been pressed
		if (isset($_POST['update_options'])) {
			if (defined('POC_CACHE')) {
				// the cache needs to be cleared since we've changed the options
				$cache = new POC_Cache();
				$cache->clear_all();
			}
			// Fill up the options array as necessary
			$options['limit'] = $_POST['limit'];
			$options['skip'] = $_POST['skip'];
			$options['trim_before'] = $_POST['trim_before'];
			$options['show_private'] = $_POST['show_private'];
			$options['show_pages'] = $_POST['show_pages'];
			$options['none_text'] = $_POST['none_text'];
			// the posted variable is an array but we want to store it as a comma separated list of values ... '9999' === no categories
			if ($_POST['excluded_cats']) {
				$options['excluded_cats'] = rtrim(implode(',', $_POST['excluded_cats']), ',');
			} else {
				$options['excluded_cats'] = '9999';
			}
			// the posted variable is an array but we want to store it as a comma separated list of values ... '9999' === no categories
			if ($_POST['excluded_authors']) {
				$options['excluded_authors'] = rtrim(implode(',', $_POST['excluded_authors']), ',');
			} else {
				$options['excluded_authors'] = '9999';
			}
			$options['excluded_posts'] = rtrim($_POST['excluded_posts']);
			// this is a complicated option to store ... two matched arrays of tags
			$st = explode("\n", trim($_POST['starttags']));
			$se = explode("\n", trim($_POST['endtags']));
			if (count($st) != count($se)) {
				$options['stripcodes'] = array(array());
			} else {
				$num = count($st);
				for ($i = 0; $i < $num; $i++) {
					$options['stripcodes'][$i]['start'] = $st[$i];
					$options['stripcodes'][$i]['end'] = $se[$i];
				}
			}

			$options['prefix'] = $_POST['prefix'];
			$options['suffix'] = $_POST['suffix'];
			$options['output_template'] = $_POST['output_template'];
			
			//options just for Similar Posts
			$options['match_cat'] = $_POST['match_cat'];
			$options['bias_title'] = $_POST['bias_title'];
			$options['utf8'] = $_POST['utf8'];
			$options['num_terms'] = $_POST['num_terms'];
			if ($options['num_terms'] < 1) $options['num_terms'] = 20;

			// store the option values under the plugin filename
			update_option(basename(__FILE__, ".php"), $options);
			
			// Show a message to say we've done something
			echo '<div class="updated"><p>' . __('Options saved') . '</p></div>';
		} else {
			// If we are just displaying the page we first load up the options array
			$options = get_option(basename(__FILE__, ".php"));
		}
		//now we drop into html to display the option page form
		?>
			<div class="wrap">
			<h2><?php echo ucwords(str_replace('-', ' ', basename(__FILE__, ".php"). __(' Options'))); ?></h2>
			<h3><a href="http://rmarsh.com/plugins/post-options/">Help and Instructions</a></h3>
			<form method="post" action="">
			<fieldset class="options">
			<table class="optiontable">
				<tr valign="top">
					<th scope="row"><?php _e('Number of posts to show:') ?></th>
					<td><input name="limit" type="text" id="limit" value="<?php echo $options['limit']; ?>" size="2" /></td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Number of posts to skip:') ?></th>
					<td><input name="skip" type="text" id="skip" value="<?php echo $options['skip']; ?>" size="2" /></td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Show password-protected posts?') ?></th>
					<td>
	            	<select name="show_private" id="show_private">
	              	<option <?php if($options['show_private'] == 'false') { echo 'selected="selected"'; } ?> value="false">No</option>
	              	<option <?php if($options['show_private'] == 'true') { echo 'selected="selected"'; } ?> value="true">Yes</option>
	            	</select> 
					</td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Show static pages?') ?></th>
	          		<td>
	        			<select name="show_pages" id="show_pages">
	        	  		<option <?php if($options['show_pages'] == 'false') { echo 'selected="selected"'; } ?> value="false">No</option>
						<option <?php if($options['show_pages'] == 'true') { echo 'selected="selected"'; } ?> value="true">Yes</option>
						</select>
					</td> 
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Limit matches to same category?') ?></th>
	          		<td>
	        			<select name="match_cat" id="match_cat">
	        	  		<option <?php if($options['match_cat'] == 'false') { echo 'selected="selected"'; } ?> value="false">No</option>
						<option <?php if($options['match_cat'] == 'true') { echo 'selected="selected"'; } ?> value="true">Yes</option>
						</select>
					</td> 
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Maximum number of words to use for match:') ?></th>
					<td><input name="num_terms" type="text" id="num_terms" value="<?php echo $options['num_terms']; ?>" size="3" /></td>
				</tr>
				<tr valign="top">
					<th scope="row" title=""><?php _e('Relative weight of post title vs. content:') ?></th>
	          		<td>
	        			<select name="bias_title" id="bias_title">
	        	  		<option <?php if($options['bias_title'] == 'high') { echo 'selected="selected"'; } ?> value="high">High</option>
						<option <?php if($options['bias_title'] == 'equal') { echo 'selected="selected"'; } ?> value="equal">Equal</option>
						<option <?php if($options['bias_title'] == 'low') { echo 'selected="selected"'; } ?> value="low">Low</option>
						</select>
					</td> 
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Handle extended characters?') ?></th>
	          		<td>
	        			<select name="utf8" id="utf8">
	        	  		<option <?php if($options['utf8'] == 'false') { echo 'selected="selected"'; } ?> value="false">No (faster)</option>
						<option <?php if($options['utf8'] == 'true') { echo 'selected="selected"'; } ?> value="true">Yes (slower)</option>
						</select>
					</td> 
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Default display if no matches:') ?></th>
					<td><input name="none_text" type="text" id="none_text" value="<?php echo htmlspecialchars(stripslashes($options['none_text'])); ?>" size="40" /></td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Text and codes before the list:') ?></th>
					<td><input name="prefix" type="text" id="prefix" value="<?php echo htmlspecialchars(stripslashes($options['prefix'])); ?>" size="40" /></td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Text and codes after the list:') ?></th>
					<td><input name="suffix" type="text" id="suffix" value="<?php echo htmlspecialchars(stripslashes($options['suffix'])); ?>" size="40" /></td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Output template:') ?></th>
					<td><textarea name="output_template" id="output_template" rows="4" cols="60"><?php echo htmlspecialchars(stripslashes($options['output_template'])); ?></textarea></td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Text to trim at start:') ?></th>
					<td><input name="trim_before" type="text" id="trim_before" value="<?php echo $options['trim_before']; ?>" size="10" /></td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Posts to exclude:') ?></th>
					<td><input name="excluded_posts" type="text" id="excluded_posts" value="<?php echo $options['excluded_posts']; ?>" size="20" /> <?php _e('comma-separated IDs'); ?></td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Authors to exclude:') ?></th>
	          		<td>
						<table border="1">	
						<?php 
							$excluded = explode(',', $options['excluded_authors']);
							$users = $wpdb->get_results("SELECT ID, user_login FROM $wpdb->users ORDER BY user_login");
							if ($users) {
								foreach ($users as $user) {
										if (false === in_array($user->ID, $excluded)) {
											$ischecked = '';
										} else {
											$ischecked = 'checked';
										}
										echo "\n\t<tr valign=\"top\"><td>$user->user_login</td><td><input type=\"checkbox\" name=\"excluded_authors[]\" value=\"$user->ID\" $ischecked /></td></tr>";
								}
							}	
						?>
						</table>
					</td> 
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Categories to exclude:') ?></th>
	          		<td>
						<table border="1">	
						<?php 
							$excluded = explode(',', $options['excluded_cats']);
							SimilarPosts::display_cats($excluded, 'excluded_cats'); 
						?>
						</table>
					</td> 
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Other plugins\' tags to remove from snippet:') ?></th>
	          		<td>
						<table>	
						<tr><td><?php _e('opening') ?></td><td><?php _e('closing') ?></td></tr>
						<tr valign="top"><td><textarea name="starttags" id="starttags" rows="4" cols="10"><?php foreach ($options['stripcodes'] as $tag) echo htmlspecialchars(stripslashes($tag['start']))."\n"; ?></textarea></td><td><textarea name="endtags" id="endtags" rows="4" cols="10"><?php foreach ($options['stripcodes'] as $tag) echo htmlspecialchars(stripslashes($tag['end']))."\n"; ?></textarea></td></tr>
						</table>
					</td> 
				</tr>
			</table>
			</fieldset>
			<div class="submit"><input type="submit" name="update_options" value="<?php _e('Update') ?>"  style="font-weight:bold;" /></div>
			</form>    		
		</div>
		<?php	
	}

	/*
		This very untidy function is called recursively from the options page above to display an indented table of categories and checkboxes
		If it didn't have to call itself it would be embedded in the page
		$excluded is the array of categories to exclude 
	*/
	function display_cats($excluded, $po_name, $currentcat = 0, $currentparent = 0, $parent = 0, $level = 0, $categories = 0) {
		global $wpdb, $bgcolor;
		if (!$categories) {
			$categories = $wpdb->get_results("SELECT * FROM $wpdb->categories WHERE category_count > 0 ORDER BY cat_name");
		}
		if ($categories) {
			foreach ($categories as $category) {
				if ($currentcat != $category->cat_ID && $parent == $category->category_parent) {
					$count = $wpdb->get_var("SELECT COUNT(post_id) FROM $wpdb->post2cat WHERE category_id = $category->cat_ID");
					$pad = str_repeat('&#8211; ', $level);
					$category->cat_name = wp_specialchars($category->cat_name);
					if (false === in_array($category->cat_ID, $excluded)) {
						$ischecked = '';
					} else {
						$ischecked = 'checked';
					}
					echo "\n\t<tr valign=\"top\"><td>$pad$category->cat_name</td><td><input type=\"checkbox\" name=\"".$po_name."[]\" value=\"$category->cat_ID\" $ischecked /></td></tr>";
					SimilarPosts::display_cats($excluded, $po_name, $currentcat, $currentparent, $category->cat_ID, $level +1, $categories);
				}
			}
		} else {
			return false;
		}
	}

}

function get_words($string, $utf8) {
	//tidy up the string a little 
	$string = addslashes(strip_tags($string));
	//make an array of all the words in the string (because we have used addslashes above this will chop off apostrophised endings)
	if ($utf8) {
		// handle extended characters...
		if (function_exists('mb_strtolower')) {
			// we can do it two ways: first if the mb functions are available ...
			@ $string = mb_strtolower($string, mb_detect_encoding($string));
			mb_regex_encoding("utf-8");
			$wordlist = mb_split("\W+" ,$string);
		} else {
			// ... a less reliable way
			preg_match_all('/\w+/u', $string, $matches);
			$wordlist = $matches[0];
		}
	} else {
		// plain ordinary PHP function
		$wordlist = str_word_count(strtolower($string), 1);
	}
	return $wordlist;
}

/*
	Takes a string, strips it of html, and produces list of the 20 most used words (common words ignored)
*/
function get_terms($content, $title, $utf8 = false, $biastitle = 'equal', $num_terms = 20) {
	global $overusedwords;
	if ($biastitle === 'equal') {
		$string = $content . ' ' . $title;
	} else {
		$string = $content;
	}
	$wordlist = get_words($string, $utf8);
 	//count them, word=>count
	$wordtable = array_count_values($wordlist);
	//knock out the noise words ... the $overusedwords array was loaded from a file to allow for different languages
	foreach ($overusedwords as $word) {
		unset($wordtable[$word]); 
	}
	// knock out words of three or less characters since mysql ignores them for full text searches
	foreach ($wordtable as $word => $freq) {
		if (strlen($word) < 4) {
			unset($wordtable[$word]);
		}
	}
	//sort by count
	arsort($wordtable);
	
	//convert the most used words into a list 'term1 term1 term1 term2 term2 term3' etc.
	if ($num_terms < 1) $num_terms = 1;
	$terms = '';
	$num = 0;
	foreach ($wordtable as $word => $count) {
		for ($i = 1; $i <= $count; $i++) {
			$terms .= ' ' . $word;
		}
		$num++;
		if ($num >= $num_terms) break;
	}
	$terms = ltrim($terms);
	
	if ($biastitle === 'high') {
		$titlewords = get_words($title, $utf8);
		$terms .= ' ' . implode(' ', $titlewords);
	}
	return addslashes($terms);
}

/*
	Given the ID of a post, gets its most common words and stores them in a custom field called 'similarterms'
*/
function save_similar_terms($postID) {
	global $wpdb;
	//get the post content and title
	$content = $wpdb->get_row("SELECT post_content, post_title FROM $wpdb->posts WHERE ID = $postID", ARRAY_A);
	//extract its terms
	$options = get_option(basename(__FILE__, ".php"));
	$terms = get_terms($content['post_content'], $content['post_title'], $options['utf8'] == 'true', $options['bias_title'], $options['num_terms']);
	//check to see if the field is set
	$metaid = $wpdb->get_var("SELECT meta_id FROM $wpdb->postmeta WHERE post_id=$postID AND meta_key='similarterms' limit 1");
	//then insert if empty
	if (is_null($metaid)) {
		$wpdb->query("INSERT INTO $wpdb->postmeta (post_id, meta_key, meta_value) VALUES ($postID, 'similarterms', \"$terms\")");
	} 
	return $postID;
}

/*
	Given the ID of a post delete its custom field, 'similarterms'
*/
function delete_similar_terms($postID) {
	global $wpdb;
	//check to see if that post has its custom field set
	$metaid = $wpdb->get_var("SELECT meta_id FROM $wpdb->postmeta WHERE post_id=$postID AND meta_key='similarterms' limit 1");
	if (!is_null($metaid)) {
		//then delete the custom field
		$wpdb->query("DELETE FROM $wpdb->postmeta WHERE post_id = $postID AND meta_key = 'similarterms'");
	}
	return $postID;
}

/*
	Work through every post and make sure it has its custom field set
*/
function save_similar_terms_for_all_posts () {
	global $wpdb;
	$termcount = 0;
	//get every post's ID
	$postIDs = $wpdb->get_results("SELECT ID FROM $wpdb->posts", ARRAY_A);
	if (!is_null($postIDs)) {
		$options = get_option(basename(__FILE__, ".php"));
		//for each post
		foreach ($postIDs as $pID) {
			$postID = $pID['ID'];
			//check if it already has its custom field
			$metaid = $wpdb->get_var("SELECT meta_id FROM $wpdb->postmeta WHERE post_id=$postID AND meta_key='similarterms' limit 1");
			if (is_null($metaid)) {
				//if not process the content for similar terms
				$content = $wpdb->get_row("SELECT post_content, post_title FROM $wpdb->posts WHERE ID = $postID", ARRAY_A);
				$terms = get_terms($content['post_content'], $content['post_title'], $options['utf8'] == 'true', $options['bias_title'], $options['num_terms']);
				//and store them
				$wpdb->query("INSERT INTO $wpdb->postmeta (post_id, meta_key, meta_value) VALUES ($postID, 'similarterms', \"$terms\")");
				$termcount = $termcount + 1;
			}
		}
	}
	return $termcount;
}

/*
	Remove ALL similarterms custom fields
*/
function clear_similar_terms_for_all_posts () {
	global $wpdb;
	$numdeleted = $wpdb->query("DELETE FROM $wpdb->postmeta WHERE meta_key = 'similarterms'");
	if (!$numdeleted) {
		$numdeleted = 0;
	}
	return $numdeleted;
}

/*
	Import all Related Posts 'keywords' into 'similarterms'
*/
function import_related_posts_keywords () {
	global $wpdb;
	$postcount = 0;
	$keycount = 0;
	//get every post's ID
	$postIDs = $wpdb->get_results("SELECT ID FROM $wpdb->posts", ARRAY_A);
	if (!is_null($postIDs)) {
		//for each post
		foreach ($postIDs as $pID) {
			$postID = $pID['ID'];
			//get the custom 'keyword' field if any
			$keywords = $wpdb->get_results("SELECT meta_value FROM $wpdb->postmeta WHERE post_id=$postID AND meta_key='keyword'", ARRAY_A);
			if (!empty($keywords)) {
				$postcount = $postcount + 1;
				$keycount = $keycount + count($keywords);
				$terms = '';
				foreach ($keywords as $keywordinfo) {
					$terms = $terms . $keywordinfo['meta_value']. ' ';
				}
				$terms = trim(addslashes($terms));
				//check if it already has its custom field
				$metaid = $wpdb->get_var("SELECT meta_id FROM $wpdb->postmeta WHERE post_id=$postID AND meta_key='similarterms' limit 1");
				if (is_null($metaid)) {
					$wpdb->query("INSERT INTO $wpdb->postmeta (post_id, meta_key, meta_value) VALUES ($postID, 'similarterms', \"$terms\")");
				} else {
					$wpdb->query("UPDATE $wpdb->postmeta SET meta_value = \"$terms\" WHERE post_ID = $postID AND meta_key = 'similarterms'");
				}
			}
		}
	}
	$counter = array('posts' => $postcount, 'keywords' => $keycount);
	return $counter;
}


function similar_posts_manage_page() {
	global $wpdb, $table_prefix;
	// we respoond to the correct button click...
	// Handle the bulk processing of similar terms
	if (isset($_POST['process_all'])) {
		//make sure we do this just the once per press
		unset($_POST['process_all']);
		//do the dirty work
		$termcount = save_similar_terms_for_all_posts ();
		//show a message
		echo '<div class="updated"><p>' . __('Saved terms for ') . $termcount . __(' posts.') . '</p></div>';
     }
	// Handle the clearing of all similar terms
	if (isset($_POST['clear_all'])) {
		//make sure we do this just the once per press
		unset($_POST['clear_all']);
		//do the dirty work
		$numkeys = clear_similar_terms_for_all_posts ();
		//show a message
		echo '<div class="updated"><p>' . __('Cleared ') . $numkeys . __(' keys') .'</p></div>';
     }
	// Handle the import of keywords from the Related Posts plugin
	if (isset($_POST['import_all'])) {
		//make sure we do this just the once per press
		unset($_POST['import_all']);
		//do the dirty work
		$counter = import_related_posts_keywords ();
		//show a message
		echo '<div class="updated"><p>' . __('Imported ') . $counter['keywords'] . __(' keywords from ') . $counter['posts'] . __(' posts.') .'</p></div>';
     }
	 // Handle setting the collation sequence
	if (isset($_POST['set_collation'])) {
		//make sure we do this just the once per press
		unset($_POST['set_collation']);
		//do the dirty work
		if ('utf-8' === get_bloginfo('charset')) {
			// whatever the character set used by MySQL WordPress has been storing utf-8 data
			// if we do a straight conversion to utf-8 in MySQL it will mess up our data so first
			// we have to convert it to a binary format and only then into utf-8
			// Unfortunately, we have to drop the fulltext index before we convert to binary...
			if (false === ($wpdb->query("DROP INDEX post_similar ON $wpdb->posts"))) {
				echo '<div class="updated"><p>' . __('Error Dropping Index') .'</p></div>';
			} elseif (false === $wpdb->query("ALTER TABLE $wpdb->posts MODIFY post_content BLOB, MODIFY post_content BLOB")) { 		
				echo '<div class="updated"><p>' . __('ErrorNot Changed to Blob') .'</p></div>';
			} elseif (false === $wpdb->query("ALTER TABLE $wpdb->posts CHANGE post_content post_content LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL, CHANGE post_title post_title TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL")) { 		
				echo '<div class="updated"><p>' . __('Error Collation Not Changed to UTF-8') .'</p></div>';
			} elseif (false === $wpdb->query("ALTER TABLE $wpdb->posts ADD FULLTEXT post_similar ( post_title , post_content )")){
				echo '<div class="updated"><p>' . __('Error adding index') .'</p></div>';
			} else {
				echo '<div class="updated"><p>' . __('Collation Changed to UTF-8') .'</p></div>';
			}		
		} else {
			// if WordPress is using another charset do you really want to be messing around? 
			if (false === $wpdb->query("ALTER TABLE $wpdb->posts CHANGE post_content post_content LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL, CHANGE post_title post_title TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL")) { 		
				echo '<div class="updated"><p>' . __('Collation Not Changed to UTF-8') .'</p></div>';
			} else {
				echo '<div class="updated"><p>' . __('Collation Changed to UTF-8') .'</p></div>';
			}	
		}
     }
	
	?>
    <div class="wrap"> 
		<h2><?php _e('Manage Similar Posts Custom Fields') ?></h2>
		<h3><a href="http://rmarsh.com/plugins/post-options/">Help and Instructions</a></h3>
		<form method="post" action="">		
		<p><?php _e('By default this plugin processes every post each time it is viewed to find the terms to match. 
		This adds an avoidable overhead which can be bypassed if the pre-processed terms are stored
		in a custom field (called "similarterms"). This is done automatically whenever a post is published or edited
		but you can also opt to pre-process every post at once by pressing the <strong>Process</strong> 
		button below. It may take half a second per post. It will not overwrite any values that have 
		already been stored.') ?></p>
		<div class="submit">	
		<input type="submit" name="process_all" value="<?php _e('Process') ?>" style="font-weight:bold;" />
		</div>		
		<p><?php _e('There is also the option to clear <em>all</em> Similar Posts\' custom fields by pressing 
		the <strong>Clear</strong> button below.') ?></p>
		<div class="submit">
		<input type="submit" name="clear_all" value="<?php _e('Clear') ?>" style="font-weight:bold;" />
		</div> 		
		<p><?php _e('If you have been using the Related Posts plugin and have created keywords to
		sharpen up your matches you can import all your keywords into Similar Posts. This 
		overwrites the corresponding Similar Posts custom fields. To begin the import press the <strong>Import</strong> 
		button below.') ?></p>
		<div class="submit">
		<input type="submit" name="import_all" value="<?php _e('Import') ?>" style="font-weight:bold;" />
		</div>   
		<!--<p><?php _e('Although most blogs are set up to use the UTF-8 character set,
		often the underlying MySQL is not set to match. Similar Posts 
		sometimes fails to work with non-European languages because the character set and
		<em>collation</em>, the way MySQL orders characters, are not set appropriately. 
		This requires MySQL version 4.1 or above. Press the <strong>Set Collation</strong> button below to change the character set 
		to "utf8" and the collation to "utf8_general_ci". It might be good to <strong>make a backup</strong> first!
		These settings seem to work well both for European languages and for languages like Hindi,
		Greek, Arabic, and Russian. I am not sure how well this will work for languages like Chinese,
		or Japanese which are not "word-based" -- perhaps a native speaker could let me know. 
		Remember to also switch on extended character handling on the Options page.') ?></p>
		<div class="submit">
		<input type="submit" name="set_collation" value="<?php _e('Set Collation') ?>" style="font-weight:bold;" />
		</div>      --> 
		</form>       
    </div>
	<?php
}

?>